;********************************************************
;
;                The LED flash control
;
;                                 Device : PIC16F84A
;                                 Clock  : 10MHz
;                                 Author : Seiichi Inoue
;********************************************************

        processor       16f84A
        list            f=inhx32
        __config _hs_osc & _wdt_off & _pwrte_on & _cp_off
        include         p16f84a.inc
        radix           dec
        errorlevel      -302    ;Eliminate bank warning

;****************  Label Definition  ********************
ra0     equ     00              ;RA0 bit
ra1     equ     01              ;RA1 bit
ra2     equ     02              ;RA2 bit
ra3     equ     03              ;RA3 bit
ra4     equ     04              ;RA4 bit

        cblock  h'c'
pntsv                           ;Pattern save area
t100ms                          ;100ms counter
acnt                            ;Actual conuter
dindex                          ;Display index
dcnt                            ;Display conuter
d1                              ;Display data 1
d2                              ;Display data 2
d3                              ;Display data 3
d4                              ;Display data 4
d5                              ;Display data 5
d6                              ;Display data 6
d7                              ;Display data 7
d8                              ;Display data 8
d9                              ;Display data 9
d10                             ;Display data 10
d11                             ;Display data 11
d12                             ;Display data 12
d13                             ;Display data 13
d14                             ;Display data 14
        endc
dcntadr equ     d1              ;Display data address

;*************  Pattern Data Definition  ****************
;                 '1':OFF   '0':ON

;******  Pattern 0  ******
p01     equ     b'01111111'
p02     equ     b'10111111'
p03     equ     b'11011111'
p04     equ     b'11101111'
p05     equ     b'11110111'
p06     equ     b'11111011'
p07     equ     b'11111101'
p08     equ     b'11111110'
p09     equ     b'11111111'

;******  Pattern 1  ******
p11     equ     b'11100111'
p12     equ     b'11011011'
p13     equ     b'10111101'
p14     equ     b'01111110'
p15     equ     b'11111111'

;******  Pattern 2  ******
p21     equ     b'01111111'
p22     equ     b'10111111'
p23     equ     b'11011111'
p24     equ     b'11101111'
p25     equ     b'11110111'
p26     equ     b'11111011'
p27     equ     b'11111101'
p28     equ     b'11111110'
p29     equ     b'11111101'
p210    equ     b'11111011'
p211    equ     b'11110111'
p212    equ     b'11101111'
p213    equ     b'11011111'
p214    equ     b'10111111'


;******  Pattern 3  ******
p31     equ     b'01111110'
p32     equ     b'10111101'
p33     equ     b'11011011'
p34     equ     b'11100111'
p35     equ     b'11111111'

;******  Pattern 4  ******
p41     equ     b'11111110'
p42     equ     b'11111101'
p43     equ     b'11111011'
p44     equ     b'11110111'
p45     equ     b'11101111'
p46     equ     b'11011111'
p47     equ     b'10111111'
p48     equ     b'01111111'
p49     equ     b'11111111'

;****************  Program Start  ***********************
        org     0               ;Reset Vector
        goto    main
        org     4               ;Interrupt Vector
        goto    led_disp

;****************  Initial Process  *********************
main
        bsf     status,rp0      ;Change to Bank1 
        movlw   h'ff'           ;Set input mode data
        movwf   trisa           ;Set PORTA to Input mode
        clrf    trisb           ;Set PORTB to Output mode
        movlw   b'00000011'     ;PS=1:16
        movwf   option_reg      ;Set OPTION_REG
        bcf     status,rp0      ;Change to Bank0
        movlw   d'100'          ;256-1000us/6.4us=100
        movwf   tmr0            ;Set TMR0
        movlw   100
        movwf   t100ms          ;Set 100ms conut

        call    ptn2            ;No. Set pattern 2 data

        movlw   b'10100000'     ;GIE,TOIE=1
        movwf   intcon          ;Interruption enable

;****************  Key Scan Process  ********************
pt0     btfss   porta,ra0       ;RA0 ON(Low lebel) ?
        goto    lp0             ;Yes. Pattern 0
pt1     btfss   porta,ra1       ;RA1 ON ?
        goto    lp1             ;Yes. Pattern 1
pt2     btfss   porta,ra2       ;RA2 ON ?
        goto    lp2             ;Yes. Pattern 2
pt3     btfss   porta,ra3       ;RA3 ON ?
        goto    lp3             ;Yes. Pattern 3
pt4     btfss   porta,ra4       ;RA4 ON ?
        goto    lp4             ;Yes. Pattern 4
        goto    pt0             ;Repeat
lp0
        movfw   pntsv           ;Read executing pattern
        sublw   0               ;Pattern check
        btfss   status,z        ;Pattern 0 ?
        call    ptn0            ;No. Set pattern 0 data
        goto    pt0             ;Yes. Nop
lp1
        movfw   pntsv           ;Read executing pattern
        sublw   1               ;Pattern check
        btfss   status,z        ;Pattern 1 ?
        call    ptn1            ;No. Set pattern 1 data
        goto    pt1             ;Yes. Nop
lp2
        movfw   pntsv           ;Read executing pattern
        sublw   2               ;Pattern check
        btfss   status,z        ;Pattern 2 ?
        call    ptn2            ;No. Set pattern 2 data
        goto    pt2             ;Yes. Nop
lp3
        movfw   pntsv           ;Read executing pattern
        sublw   3               ;Pattern check
        btfss   status,z        ;Pattern 3 ?
        call    ptn3            ;No. Set pattern 3 data
        goto    pt3             ;Yes. Nop
lp4
        movfw   pntsv           ;Read executing pattern
        sublw   4               ;Pattern check
        btfss   status,z        ;Pattern 4 ?
        call    ptn4            ;No. Set pattern 4 data
        goto    pt4             ;Yes. Nop

;***********  Pattern 0 data set Subroutine *************
ptn0
        movlw   p01             ;Read data 1
        movwf   d1              ;Set data 1
        movlw   p02             ;Read data 2
        movwf   d2              ;Set data 2
        movlw   p03             ;Read data 3
        movwf   d3              ;Set data 3
        movlw   p04             ;Read data 4
        movwf   d4              ;Set data 4
        movlw   p05             ;Read data 5
        movwf   d5              ;Set data 5
        movlw   p06             ;Read data 6
        movwf   d6              ;Set data 6
        movlw   p07             ;Read data 7
        movwf   d7              ;Set data 7
        movlw   p08             ;Read data 8
        movwf   d8              ;Set data 8
        movlw   p09             ;Read data 9
        movwf   d9              ;Set data 9
        movlw   0
        movwf   pntsv           ;Set pattern 0
        movlw   9
        movwf   dcnt            ;Set data length
        movwf   acnt            ;Set control conter
        clrf    dindex          ;Clear display index
        return

;***********  Pattern 1 data set Subroutine *************
ptn1
        movlw   p11             ;Read data 1
        movwf   d1              ;Set data 1
        movlw   p12             ;Read data 2
        movwf   d2              ;Set data 2
        movlw   p13             ;Read data 3
        movwf   d3              ;Set data 3
        movlw   p14             ;Read data 4
        movwf   d4              ;Set data 4
        movlw   p15             ;Read data 5
        movwf   d5              ;Set data 5
        movlw   1
        movwf   pntsv           ;Set pattern 1
        movlw   5
        movwf   dcnt            ;Set data length
        movwf   acnt            ;Set control conter
        clrf    dindex          ;Clear display index
        return

;***********  Pattern 2 data set Subroutine *************
ptn2
        movlw   p21             ;Read data 1
        movwf   d1              ;Set data 1
        movlw   p22             ;Read data 2
        movwf   d2              ;Set data 2
        movlw   p23             ;Read data 3
        movwf   d3              ;Set data 3
        movlw   p24             ;Read data 4
        movwf   d4              ;Set data 4
        movlw   p25             ;Read data 5
        movwf   d5              ;Set data 5
        movlw   p26             ;Read data 6
        movwf   d6              ;Set data 6
        movlw   p27             ;Read data 7
        movwf   d7              ;Set data 7
        movlw   p28             ;Read data 8
        movwf   d8              ;Set data 8
        movlw   p29             ;Read data 9
        movwf   d9              ;Set data 9
        movlw   p210            ;Read data 10
        movwf   d10             ;Set data 10
        movlw   p211            ;Read data 11
        movwf   d11             ;Set data 11
        movlw   p212            ;Read data 12
        movwf   d12             ;Set data 12
        movlw   p213            ;Read data 13
        movwf   d13             ;Set data 13
        movlw   p214            ;Read data 14
        movwf   d14             ;Set data 14
        movlw   2
        movwf   pntsv           ;Set pattern 2
        movlw   14
        movwf   dcnt            ;Set data length
        movwf   acnt            ;Set control conter
        clrf    dindex          ;Clear display index
        return

;***********  Pattern 3 data set Subroutine *************
ptn3
        movlw   p31             ;Read data 1
        movwf   d1              ;Set data 1
        movlw   p32             ;Read data 2
        movwf   d2              ;Set data 2
        movlw   p33             ;Read data 3
        movwf   d3              ;Set data 3
        movlw   p34             ;Read data 4
        movwf   d4              ;Set data 4
        movlw   p35             ;Read data 5
        movwf   d5              ;Set data 5
        movlw   3
        movwf   pntsv           ;Set pattern 3
        movlw   5
        movwf   dcnt            ;Set data length
        movwf   acnt            ;Set control conter
        clrf    dindex          ;Clear display index
        return

;***********  Pattern 4 data set Subroutine *************
ptn4
        movlw   p41             ;Read data 1
        movwf   d1              ;Set data 1
        movlw   p42             ;Read data 2
        movwf   d2              ;Set data 2
        movlw   p43             ;Read data 3
        movwf   d3              ;Set data 3
        movlw   p44             ;Read data 4
        movwf   d4              ;Set data 4
        movlw   p45             ;Read data 5
        movwf   d5              ;Set data 5
        movlw   p46             ;Read data 6
        movwf   d6              ;Set data 6
        movlw   p47             ;Read data 7
        movwf   d7              ;Set data 7
        movlw   p48             ;Read data 8
        movwf   d8              ;Set data 8
        movlw   p49             ;Read data 9
        movwf   d9              ;Set data 9
        movlw   4
        movwf   pntsv           ;Set pattern 4
        movlw   9
        movwf   dcnt            ;Set data length
        movwf   acnt            ;Set control conter
        clrf    dindex          ;Clear display index
        return

;***************  LED display Subroutine ****************
led_disp
        bcf     intcon,t0if     ;Clear timer int flag
        movlw   d'100'          ;Set 1 millisecond
        movwf   tmr0            ;Set TMR0
        decfsz  t100ms,f        ;100ms ?
        goto    int_end         ;No.
        movlw   100
        movwf   t100ms          ;Set 100ms count

        movlw   dcntadr         ;Read head address
        addwf   dindex,w        ;Head address + index
        movwf   fsr             ;Set data address
        movfw   indf            ;Read data
        movwf   portb           ;Set data to PORTB
        incf    dindex,f        ;Index + 1
        decfsz  acnt,f          ;All data displaied ?
        goto    int_end         ;No.
        movfw   dcnt            ;Read pattern data length
        movwf   acnt            ;Set control conter
        clrf    dindex          ;Clear display index
int_end
        retfie

;********************************************************
;                END of LED flash control
;********************************************************
        end

